set feedback off
set echo on

drop type number table
/

drop type number_record
/

create type number_record
IS 
OBJECT 
( value number(3)
, label varchar2(50)
)
/

create type number_table
is 
table of number_record
/

create or replace
package number_api
is
   
   function get_numbers
   return number_table
   ;
   procedure i_prefer_spanish;
   procedure i_prefer_french;
   procedure i_prefer_english;
   procedure i_prefer_dutch;
   procedure i_prefer_german;

   procedure set_lower( p_lower in number);
   procedure set_upper( p_upper in number);
   procedure give_me_more;
end;
/

create or replace
package body number_api
is
   g_lower number(3):=1;
   g_upper number(3):=5;
   g_language varchar2(2):= 'en';

   type dictionary_table_type is table of varchar2(50) index by varchar2(50);
   
   type dictionary_type is table of dictionary_table_type index by varchar2(2);
   
   g_dictionary dictionary_type;
   
   procedure init_dictionary
   is
     l_dutch_dictionary dictionary_table_type;
     l_french_dictionary dictionary_table_type;
     l_german_dictionary dictionary_table_type;
     l_spanish_dictionary dictionary_table_type;
   begin
     l_dutch_dictionary('ONE'):= 'EEN';
     l_dutch_dictionary('TWO'):='TWEE';
     l_dutch_dictionary('THREE'):='DRIE';
     l_dutch_dictionary('FOUR'):= 'VIER';
     l_dutch_dictionary('FIVE'):= 'VIJF';
     l_dutch_dictionary('SIX'):= 'ZES';
     g_dictionary('nl'):=l_dutch_dictionary;
     l_german_dictionary('ONE'):= 'EINS';
     l_german_dictionary('TWO'):='ZWEI';
     l_german_dictionary('THREE'):='DREI';
     l_german_dictionary('FOUR'):= 'VIER';
     l_german_dictionary('FIVE'):= 'FNF';
     l_german_dictionary('SIX'):= 'SECHS';
     g_dictionary('ge'):=l_german_dictionary;
     l_spanish_dictionary('ONE'):= 'UNO';
     l_spanish_dictionary('TWO'):='DOS';
     l_spanish_dictionary('THREE'):='TRES';
     l_spanish_dictionary('FOUR'):= 'QUATRO';
     l_spanish_dictionary('FIVE'):= 'CINCO';
     l_spanish_dictionary('SIX'):= 'SEIS';
     g_dictionary('es'):=l_spanish_dictionary;
     l_french_dictionary('ONE'):= 'UN';
     l_french_dictionary('TWO'):='DEUX';
     l_french_dictionary('THREE'):='TROIS';
     l_french_dictionary('FOUR'):= 'QUATRE';
     l_french_dictionary('FIVE'):= 'CINQ';
     l_french_dictionary('SIX'):= 'SIX';
     g_dictionary('fr'):=l_french_dictionary;
   end init_dictionary; 


   procedure i_prefer_spanish
   is
   begin
     g_language:='es';
   end;

   procedure i_prefer_french
   is
   begin
     g_language:='fr';
   end;
   procedure i_prefer_english
   is
   begin
     g_language:='en';
   end;
   
   procedure i_prefer_DUTCH
   is
   begin
     g_language:='nl';
   end;

   procedure i_prefer_german
   is
   begin
     g_language:='de';
   end;

   procedure set_lower( p_lower in number)
   is
   begin
     g_lower:= p_lower;
   end;
   
   procedure set_upper( p_upper in number)
   is
   begin
     g_upper:= p_upper;
   end;


   procedure give_me_more
   is
   begin
     g_upper:= g_upper + 10;
   end;

   function translate(p_string in varchar2, p_language in varchar2)
   return varchar2
   is
     l_value varchar2(50):= p_string;
   begin
     if p_language = 'en' 
     then 
       l_value:= p_string; 
     else
       begin
         l_value := g_dictionary(p_language)(p_string);
       exception
         when others
         then
           l_value:= p_string;
       end;
     end if; 
     return l_value;
   end translate;

   function translate_number_table( p_number_table in number_table, p_language in varchar2)
   return number_table
   is
     l_number_table number_table:= p_number_table;
   begin
     for i in 1..p_number_table.count loop
       l_number_table(i).label:= translate( p_number_table(i).label, p_language);
     end loop;
     return l_number_table;
   end translate_number_table;  


   
   function get_numbers
   return number_table
   is
     l_number_table  number_table := number_table();
   begin
     for i in g_lower..g_upper loop
       l_number_table.extend;
       l_number_table(l_number_table.last):= number_record( i, to_char(to_date(to_char(i,'0000'),'yyyy'),'YEAR'));
     end loop;
     return translate_number_table(l_number_table, g_language);
   end get_numbers;  

begin
  init_dictionary;      
end;
/


create or replace view numbers
as
select *
from   table(number_api.get_numbers)
/

create or replace procedure I_WANT_MORE
is
begin
  number_api.give_me_more;
end;
/

create or replace procedure I_PREFER_FRENCH
is
begin
  number_api.I_PREFER_FRENCH;
end;
/

create or replace procedure I_PREFER_ENGLISH
is
begin
  number_api.I_PREFER_ENGLISH;
end;
/

create or replace procedure I_PREFER_DUTCH
is
begin
  number_api.I_PREFER_DUTCH;
end;
/

set serveroutput on

begin
   number_api.i_prefer_dutch;
   number_api.set_upper(6);
   number_api.set_lower(1);
end;
/


clear screen

select *
from   numbers
/
pause

exec i_prefer_french
pause

select *
from   numbers
/
pause

exec i_prefer_english
pause

select *
from   numbers
/
pause

exec i_want_more
pause
select *
from   numbers
/
exec i_want_more
pause
select *
from   numbers
/
exec i_want_more
pause
select *
from   numbers
/
exec i_want_more
pause
select *
from   numbers
/

prompt What is going on here????
pause

begin
   number_api.i_prefer_spanish;
   number_api.set_upper(4);
   number_api.set_lower(2);
end;
/

select *
from   table(number_api.get_numbers)
/   