set termout on
set echo on
set feedback on
clear screen

rem Clearly this will not work:

select d.dname
,      count(d.dname)
,      Sum(e.ename) 
from   dept d , emp e
where  e.deptno = d.deptno 
group 
by     d.dname

pause
/
pause

rem What we can about it is this:


select d.dname
,      count(d.dname)
,      Amisvarchar2Sum(e.ename) 
from   dept d , emp e
where  e.deptno = d.deptno 
group by d.dname

pause

rem First step: create an object that 'implements a data cartridge interface'

CREATE OR REPLACE type AmisVarchar2SumImpl as object
(
  str varchar2(500), -- The concatenated string, comma-delimited
  static function ODCIAggregateInitialize(sctx IN OUT AmisVarchar2SumImpl)
    return number,
  member function ODCIAggregateIterate(self IN OUT AmisVarchar2SumImpl,
    value IN varchar2) return number,
  member function ODCIAggregateTerminate(self IN AmisVarchar2SumImpl,
    returnValue OUT varchar2, flags IN number) return number,
  member function ODCIAggregateMerge(self IN OUT AmisVarchar2SumImpl,
    ctx2 IN AmisVarchar2SumImpl) return number
);
/

rem Then, implement the Object's Body. It must implement functions Initialize, Iterate and Terminate
pause

CREATE OR REPLACE type body AmisVarchar2SumImpl is 
static function ODCIAggregateInitialize(sctx IN OUT AmisVarchar2SumImpl) 
return number is 
begin
  sctx := AmisVarchar2SumImpl(null);
  return ODCIConst.Success;
end;
member function ODCIAggregateIterate(self IN OUT AmisVarchar2SumImpl, value IN varchar2) return number is
begin
  case when self.str is not null
	      then self.str := self.str||', '||value;
	 else  self.str := value;
  end case;
  return ODCIConst.Success;
end;
member function ODCIAggregateTerminate(self IN AmisVarchar2SumImpl
, returnValue OUT varchar2, flags IN number) return number is
begin
  returnValue := self.str;
  return ODCIConst.Success;
end;
member function ODCIAggregateMerge(self IN OUT AmisVarchar2SumImpl, ctx2 IN AmisVarchar2SumImpl) return number is
begin
  return ODCIConst.Success;
end;
end;
/

rem Now create a function AMISVarchar2Sum that wraps the Object Aggregating Capabilites
pause

CREATE OR REPLACE FUNCTION AMISVarchar2Sum (input varchar2) RETURN varchar2
PARALLEL_ENABLE AGGREGATE USING AmisVarchar2SumImpl
;
/
rem Now we are all set to SUM VARCHAR2 values:

pause
select d.dname
,      count(d.dname)
,      Amisvarchar2Sum(e.ename) 
from   dept d , emp e
where  e.deptno = d.deptno 
group 
by     d.dname

pause
/

pause
select Amisvarchar2Sum(dname) 
from   dept d 
/
pause

drop type AmisVarchar2SumImpl
/
drop function AMISVarchar2Sum
/
clear screen