set termout on
set echo on
set output on
set feedback on

clear screen

select ename
,      sal
from   emp
order
by     sal desc
/

Prompt The Top 3 Best Paid Employees?

select ename
,      sal
from   emp
where  rownum < 4
order
by     sal desc

pause
/

rem the correct answer, using an In Line View:
pause


select empno
,      ename
,      sal
,      deptno
from   ( select e.* 
         from   emp e 
         order 
         by     sal desc
       ) 
where  rownum < 4

pause
/
Rem And using the WITH clause

pause

with ordered_emps as
     ( select empno
       ,      ename
       ,      sal
       ,      deptno
       from   emp
       order
       by     sal desc
     )
select *
from   ordered_emps 
where  rownum < 4

pause
/
pause

clear screen

select ename
,      deptno
,      sal
from   emp
order
by     deptno
,      sal desc
/

Prompt the Top 3 Earners Per Department

pause

Rem Using the Analytical Function rank() over( partition by)

select *
from  ( select ename, deptno, sal
        ,      rank() over ( partition by deptno 
                             order by sal desc) rnk 
        from   emp
      )
where rnk <4
order
by    deptno

pause
/

pause
REM Query C will do the trick as well:

with ordered_emps as
( select e.ename
  ,      e.deptno
  ,      e.sal
  from   emp e 
  order 
  by     sal desc)
select *
from   ordered_emps oe
where  oe.deptno = 10
and    rownum < 4
union all 
select *
from   ordered_emps oe
where  oe.deptno = 20
and    rownum < 4
union all 
select *
from   ordered_emps oe
where  oe.deptno = 30
and    rownum < 4

pause
/

pause

rem what about query A:

with ordered_emps as
( select e.ename
  ,      e.deptno
  ,      e.sal
  from   emp e 
  order 
  by     sal desc)
select *
from   emp
       natural join
       dept
where  empno in ( select empno 
                  from   ( select oe.*
                           ,      rownum rn
                           from   ordered_emps oe
                           where  oe.deptno = deptno
                         )       
                  where  rn < 4
                )

pause 

/                

rem So, Analytical Functions are your friend in times like these!

rem what about for example finding the average salary in the same job_category,
rem the number of colleagues with the same job
rem and the average salary in the department for each employee
rem Oh. And the difference between the employees salary and the next one down on the department's salary ranking

select *
from  ( select ename
        ,      deptno
        ,      job
        ,      sal
        ,      rank()     over ( partition by deptno 
                                 order by sal desc)   rnk 
        ,      avg(sal)   over ( partition by job)    avg_job_sal
        ,      count(job) over ( partition by job)    num_cols
        ,      avg(sal)   over ( partition by deptno) avg_deptno_salary
        ,      sal - 
               lead(sal)  over ( partition by deptno 
                                 order by sal desc)   sal_gap
        from   emp
      )
where rnk <4
order
by    deptno
,     sal desc

pause
/
